/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.cocoon.stringtemplate;

import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.antlr.stringtemplate.AttributeRenderer;
import org.antlr.stringtemplate.StringTemplate;
import org.apache.cocoon.pipeline.caching.CacheKey;
import org.apache.cocoon.pipeline.caching.CompoundCacheKey;
import org.apache.cocoon.pipeline.caching.ParameterCacheKey;
import org.apache.cocoon.pipeline.caching.TimestampCacheKey;
import org.apache.cocoon.pipeline.component.CachingPipelineComponent;
import org.apache.cocoon.pipeline.component.Starter;
import org.apache.cocoon.pipeline.component.sax.AbstractXMLProducer;
import org.apache.cocoon.servlet.controller.ControllerContextHelper;
import org.apache.cocoon.sitemap.util.ExceptionHandler;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;


public class StringTemplateGenerator extends AbstractXMLProducer implements Starter, CachingPipelineComponent {

    private URL source;
    private Map<String, Object> parameters = new HashMap<String, Object>();

    public StringTemplateGenerator() {
        super();
    }

    public StringTemplateGenerator(URL source) {
        super();
        this.source = source;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.apache.cocoon.pipeline.component.CachingPipelineComponent#constructCacheKey()
     */
    public CacheKey constructCacheKey() {
        CompoundCacheKey cacheKey = new CompoundCacheKey();

        try {
            cacheKey.addCacheKey(new TimestampCacheKey(this.source, this.source.openConnection().getLastModified()));
            cacheKey.addCacheKey(new ParameterCacheKey(this.parameters));
        } catch (IOException e) {
            e.printStackTrace();
        }

        return cacheKey;
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.apache.cocoon.pipeline.component.Starter#execute()
     */
    public void execute() {
        if (this.source == null) {
            throw new IllegalArgumentException("StringTemplateGenerator has no source.");
        }

        try {
            XMLReader xmlReader = XMLReaderFactory.createXMLReader();

            xmlReader.setContentHandler(this.getXMLConsumer());
            xmlReader.setProperty("http://xml.org/sax/properties/lexical-handler", this.getXMLConsumer());

            InputStream inputStream = this.source.openStream();
            try {
                StringTemplate stringTemplate = new StringTemplate(IOUtils.toString(inputStream, "UTF-8"));
                stringTemplate.registerRenderer(String.class, new AttributeRenderer() {
                    public String toString(Object object) {
                        return StringEscapeUtils.escapeXml(object.toString());
                    }
                });

                Map<String, Object> controllerContext = ControllerContextHelper.getContext(this.parameters);
                for (Entry<String, Object> eachEntry : controllerContext.entrySet()) {
                    stringTemplate.setAttribute(eachEntry.getKey(), eachEntry.getValue());
                }

                xmlReader.parse(new InputSource(new StringReader(stringTemplate.toString())));
            } finally {
                IOUtils.closeQuietly(inputStream);

            }
        } catch (Exception e) {
            throw ExceptionHandler.getInvocationException(e);
        }
    }

    /**
     * {@inheritDoc}
     * 
     * @see org.apache.cocoon.pipeline.component.AbstractXMLProducer#setConfiguration(java.util.Map)
     */
    @Override
    public void setConfiguration(Map<String, ? extends Object> parameters) {
        this.setSource((URL) parameters.get("source"));
    }

    @Override
    public void setup(Map<String, Object> parameters) {
        this.parameters = parameters;
    }

    public void setSource(URL source) {
        this.source = source;
    }
}
